// fun3.h - Apstraktna klasa
//        funkcija i klasa delegata.

#ifndef _fun3_h_
#define _fun3_h_

#include <iostream>
#include <utility>
using namespace std;

namespace Izvodi {
  class Delegat;  // Deklaracija klase delegata.

  class Fun {                               // DEFINICIJA KLASE FUNKCIJA:
  public:
    ~Fun() {}                               // Virtuelan destruktor.
    virtual double operator()(double x) const =0; // Vrednost funkcije.
    virtual Delegat izvod() const =0;       // Izvod funkcije.
    virtual Fun* kopija() const& =0;        // Kopija funkcije kopiranjem.
    virtual Fun* kopija() && =0;            // Kopija funkcije premetanjem.
 
  private:
    virtual void pisi(ostream& it) const =0;  // Pisanje funkcije.
    friend ostream& operator<<(ostream& it, const Fun& f)
      { f.pisi(it); return it; }
  }; // class Fun
  class Delegat: public Fun {                 // DEFINICIJA KLASE DELEGATA:
    Fun* f;                                   // Sadrana funkcija.
    void pisi(ostream& it) const override { it << *f; } // Pisanje delegata.
  public:
    Delegat(Fun* ff) { f = ff; }                     // Stvaranje delegata.
    Delegat(const Delegat& d) { f = d.f->kopija(); } // Kopirajui konstr.
    Delegat(Delegat&& d) { f = d.f; d.f = nullptr; } // Kopirajui konstr.
    ~Delegat() { delete f; }                         // Unitavanje deleg.
    Delegat& operator=(const Delegat& d) {           // Kopirajua dodela
      if (this != &d) { delete f; f = d.f->kopija(); }     // vrednosti.
      return *this;
    }
    Delegat& operator=(Delegat&& d) {                // Premetajua dodela
      if (this != &d) { delete f; f = d.f; d.f = nullptr;} // vrednosti.
      return *this;
    }
    double operator()(double x) const override       // Vrednost delegata.
      { return (*f)(x); }
    Delegat izvod() const override                   // Izvod delegata.
      { return f->izvod(); }
    Delegat* kopija() const& override       // Kopija delegata kopiranjem.
      { return new Delegat(*this); }
    Delegat* kopija() && override           // Kopija delegata premetanjem.
      { return new Delegat(move(*this)); }
  }; // class Delegat
} // namespace Izvodi

#endif

