// skladiste.h - Generika klasa skladita stvari.

#ifndef _skladiste_h_
#define _skladiste_h_

#include <iostream>
using namespace std;

class G_puno {};                     // KLASA GREAKA: Skladite je puno.
inline ostream& operator<<(ostream& it, const G_puno&)
  { return it << "*** Skladiste je puno! ***"; }

class G_prazno {};                   // KLASA GREAKA: Skladite je prazno.
inline ostream& operator<<(ostream& it, const G_prazno&)
  { return it << "*** Skladiste je prazno! ***"; }

template <typename T>               // GENERIKA KLASA SKLADITA:
class Skladiste {
  T* niz;                           // Niz stvari.
  int kap, duz;                     // Kapacitet i popunjenost niza.
  int prvi, posl;                   // Indeks prve i poslednje stvari.
  void kopiraj(const Skladiste& s); // Kopiranje u skladite.
  void premesti(Skladiste s) {      // Premetanje u skladite.
    niz = s.niz; s.niz = nullptr;
    kap = s.kap; duz = s.duz;
    prvi = s.prvi; posl = s.posl;
  }
  void brisi() { delete [] niz; }   // Oslobaanje memorije.
public:
  explicit Skladiste(int k=10) {    // Stvaranje praznog skladita.
    niz = new T [kap = k];
    prvi = posl = duz = 0;
  }
  Skladiste(const Skladiste& s) { kopiraj(s); } // Kopirajui konstruktor.
  Skladiste(Skladiste&& s) { premesti(s); }     // Premetajui konstruktor.
  ~Skladiste() { brisi(); }                     // Destruktor.
  Skladiste& operator=(const Skladiste& s) {    // Kopirajua dodela
    if (this != &s) { brisi(); kopiraj(s); }    //   vrednosti.
    return *this;
  }
  Skladiste& operator=(Skladiste&& s) {          // Premetajua dodela
    if (this != &s) { brisi(); premesti(s); }    //   vrednosti.
    return *this;
  }
  void stavi(const T& t) {                       // Stavljanje u skladite.
    if (duz == kap) throw G_puno();
    niz[posl++] = t;
    if (posl == kap) posl = 0;
    duz++;
  }
  T uzmi() {                                     // Uzimanje iz skladita.
    if (duz == 0) throw G_prazno();
    T t = niz[prvi++];
    if (prvi == kap) prvi = 0;
    duz --;
    return t;
  }
};

template <typename T>                            // METODA KLASE SKLADITA:
void Skladiste<T>::kopiraj(const Skladiste& s) { // Kopiranje u skladite.
  niz = new T [kap = s.kap];
  for (int i=0; i<kap; i++) niz[i] = s.niz[i];
  prvi = s.prvi; posl = s.posl; duz = s.duz;
}

#endif
